#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ykpers-1/ykpers.h>
#include <yubikey.h>
#include <libcryptsetup.h>

#define LUKS_DEVICE "/cdrom/squash.rootfs"
#define LUKS_NAME "data"

int unlock_luks(const char *device, const char *name, const char *key, size_t key_size) {
  struct crypt_device *cd;

  if (crypt_init(&cd, device) < 0) {
    fprintf(stderr, "Error: Cannot initialize cryptsetup\n");
    return -1;
  }

  if (crypt_load(cd, CRYPT_LUKS2, NULL) < 0) {
    fprintf(stderr, "Error: Cannot load LUKS header\n");
    crypt_free(cd);
    return -1;
  }

  if (crypt_activate_by_passphrase(cd, name, CRYPT_ANY_SLOT, key, strlen(key), CRYPT_ACTIVATE_ALLOW_DISCARDS) < 0) {
    fprintf(stderr, "Error: Cannot decrypt device\n");
    crypt_free(cd);
    return -1;
  }

  crypt_free(cd);
  return 0;
}

int main() {
  YK_KEY *yk = NULL;
  unsigned char response[64];
  unsigned int response_len = sizeof(response);
  char *hex_response = malloc(41);
  unsigned char challenge[64] = "DUMMY_CHALLENGE\n";

  if (!hex_response) {
    fprintf(stderr, "Error: Failed memory allocation\n");
    free(hex_response);
    return 1;
  }

  if (!yk_init()) {
    fprintf(stderr, "Error: Cannot initialize libykpers\n");
    free(hex_response);
    return 1;
  }

  yk = yk_open_first_key();
  if (!yk) {
    yk_release();
    free(hex_response);
    return 1;
  }

  if (!yk_challenge_response(yk, SLOT_CHAL_HMAC1, YK_FLAG_MAYBLOCK, sizeof(challenge), challenge, response_len, response)) {
    fprintf(stderr, "Error: challenge-response issue\n");
    yk_close_key(yk);
    yk_release();
    free(hex_response);
    return 1;
  }

  for (int i = 0; i < 20; i++) {
    sprintf(hex_response + (i * 2), "%02x", response[i]);
  }
  hex_response[40] = '\0';

  yk_close_key(yk);
  yk_release();

  if (unlock_luks(LUKS_DEVICE, LUKS_NAME, hex_response, strlen(hex_response)) != 0) {
    fprintf(stderr, "Error: Cannot decrypt device\n");
    return 1;
  }

  free(hex_response);
  return 0;
}
